<?php

namespace Drupal\date_all_day\Plugin\Field\FieldWidget;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\datetime_range\Plugin\Field\FieldWidget\DateRangeDefaultWidget;

/**
 * Plugin implementation of the 'daterange_all_day' widget.
 *
 * @FieldWidget(
 *   id = "daterange_all_day",
 *   label = @Translation("Date and time range with All day"),
 *   field_types = {
 *     "daterange_all_day"
 *   }
 * )
 */
class DatetimeRangeAllDayWidget extends DateRangeDefaultWidget {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    // Add All day checkbox with states api.
    $element['value']['value_all_day'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('All day'),
      '#weight' => 2,
      '#default_value' => $items->get($delta)->value_all_day,
      '#parents' => [$items->getName(), $delta, 'value_all_day'],
    ];

    $element['end_value']['end_value_all_day'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('All day'),
      '#weight' => 2,
      '#default_value' => $items->get($delta)->end_value_all_day,
      '#parents' => [$items->getName(), $delta, 'end_value_all_day'],
    ];

    $element['#attached'] = [
      'library' => [
        'date_all_day/date_all_day',
      ],
    ];

    // Set end date as optional.
    $optional_end_date = $this->getFieldSetting('optional_end_date');

    $element['end_value']['#title'] = $optional_end_date ? $this->t('End date (optional)') : $this->t('End date');
    if ($element['#required'] && $optional_end_date) {
      $element['end_value']['#required'] = FALSE;
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function validateStartEnd(array &$element, FormStateInterface $form_state, array &$complete_form) {
    $start_date = $element['value']['#value']['object'];
    $end_date = $element['end_value']['#value']['object'];

    if ($start_date instanceof DrupalDateTime) {
      if (!$this->getFieldSetting('optional_end_date') && $end_date === NULL) {
        $form_state->setError($element['end_value'], $this->t('The @title end date is required', ['@title' => $element['#title']]));
      }

      if ($end_date instanceof DrupalDateTime) {
        if ($start_date->getTimestamp() !== $end_date->getTimestamp()) {
          $interval = $start_date->diff($end_date);
          if ($interval->invert === 1) {
            $form_state->setError($element, $this->t('The @title end date cannot be before the start date', ['@title' => $element['#title']]));
          }
        }
      }
    }
  }

}
